// **************************************************************
//     Filename: wavTrigger.h
// Date Created: 2/23/2014
//
//     Comments: Robertsonics WAV Trigger serial control library
//
// Programmers: Jamie Robertson, info@robertsonics.com
//
// **************************************************************
// modify by yves cordier yves.cordier@orange.fr
// 
// **************************************************************
//
// Revision History
//
// Date      Description
// --------  -----------
// 
//
// 02/22/14  First version created.
//           LIMITATIONS: Hard-coded for AltSoftwareSerial Library.
//           Also only supports commands TO the WAV Trigger. Will
//           fix these things.
//
// 05/10/14  Tested with UNO. Added new functions for fades, cross-
//           fades and starting multiple tracks in sample sync.
//
// 04/26/15  Added support for sample-rate / pitch bend control,
//           and compile macro switches for hardware serial ports.
//
// 11/06/16  Support for v1.30 firmware features, including two-way
//           communication with track status reporting.
//
// 19/03/17  Simplifying functions using C++ facilities by yves Cordier
//           Introduction of multiples cards connecting possibilities by using
//           pointer to hardwareserial object in communication
//           The librairie could be used as is (no lines to modify) 
//           if use of hardware serial
//	         Rewriting of update function in order to corectely get message from the
//           wave card (including delays of returning informations)


#ifndef WAVTRIGGER_H
#define WAVTRIGGER_H

// added by cordier yves to be able to have all arduino definition
#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

// ==================================================================
// The following defines are used to control which serial class is
//  used. Uncomment only the one you wish to use. If all of them are
//  commented out, the library will use Hardware Serial
// #define __WT_USE_ALTSOFTSERIAL__
// 
// Modify by Cordier Yves
// 
// In case of use Harware serial, you must precise in the constructor
// which serial is connected to the wavtrigger card
// ex : waveTrigger wave = waveTrigger(&Serial,9600);
// ex : waveTrigger wave = waveTrigger(&Seria2,9600); // on Mega
// connected to harware serial port Serial<n> at the speed of 9600 bds
// speed is optional (9600 default value)
// serial is optional (&Serial default value)
//
// For compatibility, you may use the old mode by uncomment the 
// lines you wish, this old way is not recommended
// #define __WT_USE_SERIAL__
// #define __WT_USE_SERIAL1__
// #define __WT_USE_SERIAL2__
// #define __WT_USE_SERIAL3__
// ==================================================================

#define CMD_GET_VERSION					1
#define CMD_GET_SYS_INFO				2
#define CMD_TRACK_CONTROL				3
#define CMD_STOP_ALL					4
#define CMD_MASTER_VOLUME				5
#define CMD_TRACK_VOLUME				8
#define CMD_TRACK_FADE					10
#define CMD_RESUME_ALL_SYNC				11
#define CMD_SAMPLERATE_OFFSET			12
#define	CMD_TRACK_CONTROL_EX			13
#define	CMD_SET_REPORTING				14

#define TRK_PLAY_SOLO					0
#define TRK_PLAY_POLY					1
#define TRK_PAUSE						2
#define TRK_RESUME						3
#define TRK_STOP						4
#define TRK_LOOP_ON						5
#define TRK_LOOP_OFF					6
#define TRK_LOAD						7

#define	RSP_VERSION_STRING				129
#define	RSP_SYSTEM_INFO					130
#define	RSP_STATUS						131
#define	RSP_TRACK_REPORT				132

#define MAX_MESSAGE_LEN					32
#define MAX_NUM_VOICES					14
#define VERSION_STRING_LEN				21


#define SOM1	0xf0
#define SOM2	0xaa
#define EOM		0x55


#ifdef __WT_SERIAL_ASSIGNED
#include "../AltSoftSerial/AltSoftSerial.h"
#define  __WT_SERIAL_ASSIGNED__
#else
#include <HardwareSerial.h>
#endif

// obsolete define  used for compatibility

#ifdef __WT_USE_SERIAL1__
#define WTSerial Serial1
#define __WT_SERIAL_ASSIGNED__
#endif
#ifdef __WT_USE_SERIAL2__
#define WTSerial Serial2
#define __WT_SERIAL_ASSIGNED__
#endif
#ifdef __WT_USE_SERIAL3__
#define WTSerial Serial3
#define __WT_SERIAL_ASSIGNED__
#endif



class wavTrigger
{
public:
	#ifdef __WT_SERIAL_ASSIGNED__
	wavTrigger(); // __WT_SERIAL_ASSIGNED__  case 
	#else
	wavTrigger(HardwareSerial *camPort =&Serial,unsigned long baud =9600); // others cases (hardware)
	#endif
	~wavTrigger() {;}
	void start(long speed=9600);
	void update(void);
	void flush(void);
	void setReporting(bool enable);
	bool getVersion(char *pDst, int len);
	int getNumTracks(void);
	bool isTrackPlaying(int trk);
	void masterGain(int gain);
	void stopAllTracks(void);
	void resumeAllInSync(void);
	void trackPlaySolo(int trk);
	void trackPlaySolo(int trk, bool lock);
	void trackPlayPoly(int trk);
	void trackPlayPoly(int trk, bool lock);
	void trackLoad(int trk);
	void trackLoad(int trk, bool lock);
	void trackStop(int trk);
	void trackPause(int trk);
	void trackResume(int trk);
	void trackLoop(int trk, bool enable);
	void trackGain(int trk, int gain);
	void trackFade(int trk, int gain, int time, bool stopFlag);
	void samplerateOffset(int offset);
	// added by Cordier Yves
	long getComSpeed(); // may be useful to compute delay
	
private:
	void trackControl(int trk, int code);
	void trackControl(int trk, int code, bool lock);

#ifdef __WT_SERIAL_ASSIGNED__
	AltSoftSerial WTSerial;
#else
	// added by yves cordier
	HardwareSerial *_wavTrigger_Serial; // The pointer to the serial object where is connected the wavTrigger board
#endif

	uint16_t voiceTable[MAX_NUM_VOICES];
	uint8_t rxMessage[MAX_MESSAGE_LEN];
	char version[VERSION_STRING_LEN];
	uint16_t numTracks;
	uint8_t numVoices;
	uint8_t rxCount;
	uint8_t rxLen;
	bool rxMsgReady;
	bool versionRcvd;
	bool sysinfoRcvd;
	long comSpeed; // speed of communication to board
	bool setReportingFlag; 
	
};

#endif